# Generated by Django 2.2.4 on 2019-08-22 10:52

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0022_auto_20190821_2016'),
    ]

    operations = [
        migrations.AlterField(
            model_name='holidays',
            name='daytype',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='clndr.Day_types'),
        ),
    ]
